use std::fmt;
use std::fmt::Display;



/// The different available logging levels.
#[repr(u8)]
#[derive(Clone, Copy, PartialEq, PartialOrd, Eq, Ord)]
pub enum LogLevel
{
   /// Only errors will be written to the logs.
   ///
   /// This designates a serious, program
   /// stopping, error.
   ERROR,

   /// Warnings and errors will be written to
   /// the logs.
   ///
   /// Designates a problem, but it is not
   /// something that will stop the program
   /// from running.
   WARN,

   /// Info, warnings, and errors will be
   /// written to the logs.
   ///
   /// Designates useful information 
   INFO,

   /// Debug statements, info, warnings,
   /// and errors will be written to the logs.
   ///
   /// Designates debugging information.
   DEBUG,
}


impl LogLevel
{
   pub fn from_u8(lvl: u8) -> Option<LogLevel>
   {
      match lvl
      {
         0 => Some(LogLevel::ERROR),
         1 => Some(LogLevel::WARN),
         2 => Some(LogLevel::INFO),
         3 => Some(LogLevel::DEBUG),
         _ => None
      }
   }
}

impl PartialEq for LogLevel
{
   fn eq(&self, other: &LogLevel) -> bool
   {
      *self as usize == *other as usize
   }
}

impl Display for LogLevel
{
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result
   {
      match *self
      {
         LogLevel::ERROR => write!(f, "Error"),
         LogLevel::WARN => write!(f, "Warn"),
         LogLevel::INFO => write!(f, "Info"),
         LogLevel::DEBUG => write!(f, "Debug"),
      }
   }
}
